#include "headers.h"
#include "user.h"
#include "admin.h"
#include "admin-interface.h"
#include "../ui.h"

// supporting functions
static AV *privs_to_av(int privs) {
	AV *ret = newAV();
	int i;
	
	for(i = 0; priv_text[i].text != NULL; ++i) {
		if(PRIV_U_ISSET(privs,priv_text[i].priv))
			av_push(ret,new_pv(priv_text[i].text));
	}

	return ret;
}

static uint32_t av_to_privs(AV *av) {
	uint32_t ret = PRIVS_EMPTY;
	int i,j,p;

	if(av == NULL)
		return ret;
	
	for(i = 0; i < av_len(av); ++i) {
		STRLEN n_a;
		char *str = SvPV(*av_fetch(av,i,0),n_a);
		for(j = 0, p = -1; priv_text[j].text != NULL && p != -1; ++j) {
			if(strcasecmp(str,priv_text[j].text) == 0)
				p = priv_text[j].priv;
		}
		if(p != -1)
			ret |= PRIV_TO_MASK(p);
	}

	return ret;
}

static menu_t *resolv_menu(const char *path) {
	if(strcmp(path,"global") == 0) {
		menu_ref(global_menu_root);
		return global_menu_root;
	} else if(strcmp(path,"/") == 0) {
		menu_ref(main_menu_root);
		return main_menu_root;
	} else {
		return menu_resolv(path);
	}
}

static HV *menu_to_hv(menu_t *menu) {
	HV *hv;
	int i;

	if(menu == NULL)
		return (HV*)&PL_sv_undef;
	
	hv = newHV();
	hv_store(hv,"_ptr",4,newSViv((IV)menu),0);

	LOCK(menu);
	for(i = 0; i < menu->no_ents; ++i) {
		menu_t *ent = menu->ents[i];
		HV *ehv = newHV();
		hv_store(ehv,"_ptr",4,newSViv((IV)ent),0);
		hv_store(ehv,"name",4,new_pv(ent->name),0);
		hv_store(ehv,"type",4,new_pv(ent->type == ADMIN_MENU ?
					"menu" : "command"),0);
		hv_store(ehv,"min_args",7,
			newSViv(ent->min_args),0);
		hv_store(ehv,"max_args",7,
			newSViv(ent->max_args),0);
		hv_store(ehv,"log",3,
			ent->log ? &PL_sv_yes : &PL_sv_no,0);
		hv_store(ehv,"hide",4,
			ent->hide ? &PL_sv_yes : &PL_sv_no,0);
		hv_store(ehv,"privs",5,
			newRV_noinc((SV*)privs_to_av(ent->privs)),0);
		hv_store(hv,ent->name,strlen(ent->name),
			newRV_noinc((SV*)ehv),0);
	}
	UNLOCK(menu);

	return hv;
}

static HV *instance_to_hv(instance_t *ins) {
	HV *hv = newHV();

	hv_store(hv,"prompt",6,new_pv(ins->prompt),0);
	hv_store(hv,"lmenu",5,newRV_noinc((SV*)menu_to_hv(ins->lmenu)),0);
	hv_store(hv,"gmenu",5,newRV_noinc((SV*)menu_to_hv(ins->gmenu)),0);
	hv_store(hv,"data",4,newSViv((int)ins->data),0);
	if(ins->prev != NULL)
		hv_store(hv,"prev",4,
			newRV_noinc((SV*)instance_to_hv(ins->prev)),0);

	return hv;
}

static int valid_menu(menu_t *menu) {
	if(menu == NULL)
		return 0;
	if(menu->type == ADMIN_MENU)
		return 1;
	else
		return 0;
}

static menu_t *find_menu_ent(menu_t *menu, const char *ent) {	
	int i;
	for(i = 0; i < menu->no_ents; ++i) {
		if(strcmp(menu->ents[i]->name,ent) == 0) {
			menu_ref(menu->ents[i]);
			return menu->ents[i];
		}
	}
	return NULL;
}	

MODULE = Arisa::UI  PACKAGE = Arisa::UI
PROTOTYPES: ENABLE

IV
_menu_add(is_menu, path, name, min_args, max_args, log, hide, privs)
	int is_menu
	char *path
	char *name
	int min_args
	int max_args
	int log
	int hide
	SV *privs
PREINIT:
	menu_t *menu, *cmd = NULL;
CODE:
	RETVAL = 0;
	menu = resolv_menu(path);
	if(valid_menu(menu) && (cmd = find_menu_ent(menu,name)) == NULL) {
		cmd = alloc_menu(is_menu ? ADMIN_MENU : ADMIN_COMMAND);

		cmd->name	= xstrdup(name);
		cmd->func	= perl_make_ui_call;
		cmd->min_args	= min_args;
		cmd->max_args	= max_args;
		cmd->log	= log;
		cmd->hide	= hide;
		cmd->privs	= av_to_privs(avref(privs));

		menu_add(menu,cmd);
		RETVAL = (IV) cmd;
	}
	if(menu != NULL)
		menu_deref(menu);
	if(cmd != NULL)
		menu_deref(cmd);
OUTPUT:
	RETVAL

IV
_menu_del(path, name)
	char *path
	char *name
PREINIT:
	menu_t *menu;
CODE:
	RETVAL = 0;
	menu = resolv_menu(path);
	if(menu != NULL) {
		menu_t *found = NULL;
		
		LOCK(menu);
		found = find_menu_ent(menu,name);
		UNLOCK(menu);

		if(found != NULL) {
			menu_del(menu,found);
			menu_deref(found);
			RETVAL = (IV) found;
		}

		menu_deref(menu);
	}
OUTPUT:
	RETVAL

void
menu_list(path)
	char *path
PREINIT:
	HV 	*hv = NULL;
	menu_t	*menu;
PPCODE:
	menu = resolv_menu(path);
	if(menu != NULL) {
		if(menu->type == ADMIN_MENU)
			hv = menu_to_hv(menu);
		menu_deref(menu);
	}
	RET_REF(hv);

void
get_help(path)
	char *path
PREINIT:
	menu_t *cmd;
PPCODE:
	cmd = resolv_menu(path);
	if(cmd != NULL) {
		int i;
		
		LOCK(cmd);
		for(i = 0; i < cmd->no_help; ++i)
			XPUSHs(sv_2mortal(new_pv(cmd->help[i])));
		UNLOCK(cmd);
		
		menu_deref(cmd);
	}

SV*
set_help(path, ...)
	char *path
PREINIT:
	menu_t *cmd;
	int i;
CODE:
	cmd = resolv_menu(path);
	if(cmd != NULL) {
		LOCK(cmd);
		strarr_free(&(cmd->help),&(cmd->no_help));
		for(i = 1; i < items; ++i) {
			STRLEN n_a;
			char *htxt = SvPV(ST(i),n_a);
			PTRARR_ADD(&(cmd->help),&(cmd->no_help),xstrdup(htxt));
		}
		UNLOCK(cmd);
		menu_deref(cmd);
		RETVAL = &PL_sv_yes;
	} else {
		RETVAL = &PL_sv_no;
	}
OUTPUT:
	RETVAL

int
RET_OK()
CODE:
	RETVAL = RET_OK;
OUTPUT:
	RETVAL

int
RET_UP()
CODE:
	RETVAL = RET_UP;
OUTPUT:
	RETVAL

int
RET_RTN()
CODE:
	RETVAL = RET_RTN;
OUTPUT:
	RETVAL

int
RET_QIT()
CODE:
	RETVAL = RET_QIT;
OUTPUT:
	RETVAL


#*******************************
MODULE = Arisa::UI  PACKAGE = Arisa::UI::Context
#*******************************

Arisa::Admin
admin(ctx_ptr)
	Arisa::UI::Context ctx_ptr
CODE:
	RETVAL = NULL;
	if(ctx_ptr == perl_ui_valid_context) {
		acontext_t *ctx = (acontext_t *)ctx_ptr;
		int i;
		
		LOCK(global);
		for(i = 0; i < global->no_admins && RETVAL == NULL; ++i) {
			chat_t *adm = global->admins[i];
			LOCK(adm);
			if(&(adm->msgqueue) == ctx->msgqueue)
				RETVAL = adm;
			UNLOCK(adm);
		}
		UNLOCK(global);
	}
OUTPUT:
	RETVAL

SV*
colour(ctx_ptr, nc = NULL)
	Arisa::UI::Context ctx_ptr
	char *nc
CODE:
	RETVAL = &PL_sv_undef;
	if(ctx_ptr == perl_ui_valid_context) {
		acontext_t *ctx = (acontext_t *)ctx_ptr;
		if(nc != NULL) {
			int c = str_to_colourtype(nc);
			if(c != -1)
				ctx->colour = c;
		}
		RETVAL = new_pv(colourtype_to_str(ctx->colour));
	}
OUTPUT:
	RETVAL

void
instance(ctx_ptr)
	Arisa::UI::Context ctx_ptr
PREINIT:
	HV *ret = NULL;
PPCODE:
	if(ctx_ptr == perl_ui_valid_context) {
		acontext_t *ctx = (acontext_t *)ctx_ptr;
		instance_t *ins = (instance_t *)ctx->instance;
		ret = instance_to_hv(ins);
	}
	RET_REF(ret);

SV*
set_prompt(ctx_ptr, prompt = NULL)
	Arisa::UI::Context ctx_ptr
	char *prompt
CODE:
	RETVAL = &PL_sv_no;
	if(ctx_ptr == perl_ui_valid_context) {
		acontext_t *ctx = (acontext_t *)ctx_ptr;
		instance_t *ins = (instance_t *)ctx->instance;

		if(ins->prompt != NULL && ins->free_prompt != 0)
			xfree(ins->prompt);
		if(prompt != NULL) {
			ins->prompt = xstrdup(prompt);
			ins->free_prompt = 1;
		} else {
			ins->prompt = "";
			ins->free_prompt = 0;
		}
		RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL

SV*
send_msg(ctx_ptr, msg)
	Arisa::UI::Context ctx_ptr
	char *msg
CODE:
	RETVAL = &PL_sv_no;
	if(ctx_ptr == perl_ui_valid_context) {
		acontext_t *ctx = (acontext_t *)ctx_ptr;
		pqueue_push_back(ctx->msgqueue,xstrdup(msg));
		RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL

Arisa::User
user(ctx_ptr)
	Arisa::UI::Context ctx_ptr
CODE:
	RETVAL = NULL;
	if(ctx_ptr == perl_ui_valid_context) {
		acontext_t *ctx = (acontext_t *)ctx_ptr;
		RETVAL = ctx->user;
	}
OUTPUT:
	RETVAL
